#!/usr/bin/perl

use File::Copy;

# board-mod.pl v0.1 by nep@ikami.com
# Apply BoardMod patches to YaBB!

# To do: find a mod that applies patches to YaBB.(pl/cgi) and make it work for
# both- use the YaBB Settings.pl!

# Path to the yabb directory, yo
my $_yabb_path = '/path/to/yabb';

# Save patches to ".MODDED", in case something gets hosed
my $_safe_patch = 0;

# Back-up the originals? Not too smart- it won't overwrite an existing .bak
# file
my $_backup = 1;

main();

sub main {
	my $file = shift(@ARGV) || usage();

	# flags.. "remove" removes the mod, "test" enables safe patching mode..
	my %flags;
	for (@ARGV) {
		$flags{$_}=1;
	}
	$_safe_patch = ($flags{test})?1:0;

	# Load the patches from the mod file
	my @patch = parse_patch($file,$flags{remove});

	# Back the files up
	if ($_backup) {
		backup_files(@patch);
	}

	# Apply ye olde patches
	for (@patch) {
		apply_patch($_);
	}
}

sub usage {
	print "Usage: $0 mod-file [remove]\n";
	exit(0);
}

sub parse_patch {
	# Let's fetch the data into a format that we can actually use, shall we?
	my $file = shift || return;
	my $remove = shift || 0;

	my $n = 0;
	my $tag_line = 0;
	my $last_search_line = 0;
	my $edit_file;
	my $cur_tag = 0;
	my $in_tag = 0;
	my @patch;
	my @tag_data;
	my @last_search;

	open(IN,"<$file") || die "Can't read file: $!\n";

	while (my $line = <IN>) {
		$line=~s/\r?\n//;
		$n++;
		if ($in_tag) {
			if ($line=~/<\/$cur_tag>/) {
				if ($cur_tag eq 'edit file') {
					$edit_file = shift(@tag_data);
				} elsif ($cur_tag eq 'search for') {
					@last_search = @tag_data;
					$last_search_line = $tag_line;
				} else {
					my @search = @last_search;
					my @data = @tag_data;
					if ($remove) {
						@search = @tag_data;
						if ($cur_tag eq 'replace') {
							# swap the thingies
							@data = @last_search;
						} else {
							# be more violent
							$cur_tag = 'replace';
							undef @data;
						}
					}
					push(@patch,{	file	=> $edit_file,
									line	=> $last_search_line,
									search	=> \@search,
									action	=> $cur_tag,
									data	=> \@data
							});
				}
				undef $cur_tag;
				undef @tag_data;
				$in_tag = 0;
			} else {
				push(@tag_data,$line);
			}
		} else {
			if ($line=~/^<(.+)>/) {
				$cur_tag = $1;
				$tag_line = $n;
				$in_tag = 1;
			}
		}
	}

	close(IN);

	return @patch;
}

sub apply_patch {
	my $tmp = shift || return;
	my %patch = %$tmp;

	if ($patch{file} && $patch{search} && $patch{action}) {
		my $_if = sprintf("%s/%s%s",$_yabb_path,$patch{file},($_safe_patch)?'.MODDED':'');
		if ($_safe_patch && !-e $_if) {
			# There's been no modded file saved, load from the original
			$_if = sprintf("%s/%s",$_yabb_path,$patch{file});
		}
		my $_of = sprintf("%s/%s%s",$_yabb_path,$patch{file},($_safe_patch)?'.MODDED':'');

		my @search = @{$patch{search}};
		print "Patching $_if\n";

		# let's do our search, eh?
		if (-e $_if) {
			my @data;
			my $patch_found = 0;
			my $n = 0;

			open(IN,"<$_if") || die "Can't read $_if: $!\n";
			while (my $line = <IN>) {
				$line=~s/\r?\n//;
				push(@data,$line);

				if (!$patch_found) {
					if ($line eq $search[$#search]) {
						# possible match found..
						my $match=1;
						for my $i (0..$#search) {
							if ($data[$n-($#search-$i)] ne $search[$i]) {
								$match=0;
							}
						}
						if ($match) {
							$patch_found=1;
							if ($patch{action} eq 'add before') {
								splice(@data,$n-$#search,0,@{$patch{data}});
							} elsif ($patch{action} eq 'add after') {
								push(@data,@{$patch{data}});
							} elsif ($patch{action} eq 'replace') {
								if (@{$patch{data}}) {
									splice(@data,$n-$#search,($#search+1),@{$patch{data}});
								} else {
									# Empty data, we're removing a chunk o'
									# code, and don't want to slip in an extra
									# newline there..
									splice(@data,$n-$#search,($#search+1));
								}
							} else {
								print "Bad patch action on match: $patch{action}\n";
							}
						}
					}

					$n++;
				}
			}
			close(IN);

			if ($patch_found) {
				open(OUT,">$_of") || die "Can't save $_of: $!\n";
				print OUT join("\n",@data),"\n";
				close(OUT);
			} else {
				print "No match found for patch (patch line $patch{line})\n";
			}
		} else {
			print "Unable to locate patch source: $_if\n";
		}
	}
}

sub backup_files {
	for my $tmp (@_) {
		my %tmp = %$tmp;

		my $_if = sprintf("%s/%s",$_yabb_path,$tmp{file});
		my $_of = sprintf("%s/%s.bak",$_yabb_path,$tmp{file});

		if (!-e $_of) {
			copy $_if, $_of;
		}
	}
}

